/**
 * Nominally-typed primitive types.
 *
 * See:
 * - https://basarat.gitbook.io/typescript/main-1/nominaltyping#using-interfaces
 * - https://github.com/Microsoft/TypeScript/blob/7b48a182c05ea4dea81bab73ecbbe9e013a79e99/src/compiler/types.ts#L693-L698
 */
import { Opaque } from "ts-essentials";

export type AscDesc = "asc" | "desc";

export function maybe<A, B>(box: (value: B) => A, value: B | undefined): A | undefined {
  if (value === undefined) return undefined;
  return box(value);
}

export type OriginRequestPath = Opaque<string, "OriginRequestPath">;
export function OriginRequestPath(value: string): OriginRequestPath {
  return value as any;
}

export type OriginRequestTransformationHeaderValue = Opaque<string, "OriginRequestTransformationHeaderValue">;
export function OriginRequestTransformationHeaderValue(value: string): OriginRequestTransformationHeaderValue {
  return value as any;
}

export type UploadTransformerBaseUrl = Opaque<string, "UploadTransformerBaseUrl">;
export function UploadTransformerBaseUrl(value: string): UploadTransformerBaseUrl {
  return value as any;
}

export type UploadCdnBaseUrl = Opaque<string, "UploadCdnBaseUrl">;
export function UploadCdnBaseUrl(value: string): UploadCdnBaseUrl {
  return value as any;
}

export type TransformationRootPath = Opaque<string, "TransformationRootPath">;
export function TransformationRootPath(value: string): TransformationRootPath {
  return value as any;
}

export type OrganizationName = Opaque<string, "OrganizationName">;
export function OrganizationName(value: string): OrganizationName {
  return value as any;
}

export type DepartmentName = Opaque<string, "DepartmentName">;
export function DepartmentName(value: string): DepartmentName {
  return value as any;
}

export type AccountId = Opaque<string, "AccountId">;
export function AccountId(value: string): AccountId {
  return value as any;
}

export type AccountJobId = Opaque<string, "AccountJobId">;
export function AccountJobId(value: string): AccountJobId {
  return value as any;
}

export type FileId = Opaque<string, "FileId">;
export function FileId(value: string): FileId {
  return value as any;
}

export type FileUniquifier = Opaque<string, "FileUniquifier">;
export function FileUniquifier(value: string): FileUniquifier {
  return value as any;
}

export type UserId = Opaque<string, "UserId">;
export function UserId(value: string): UserId {
  return value as any;
}

export type MailChimpContactId = Opaque<string, "MailChimpContactId">;
export function MailChimpContactId(value: string): MailChimpContactId {
  return value as any;
}

export type PaddleSubscriptionId = Opaque<number, "PaddleSubscriptionId">;
export function PaddleSubscriptionId(value: number): PaddleSubscriptionId {
  return value as any;
}

export type PaddleVendorId = Opaque<number, "PaddleVendorId">;
export function PaddleVendorId(value: number): PaddleVendorId {
  return value as any;
}

export type PaddleAuthCode = Opaque<string, "PaddleAuthCode">;
export function PaddleAuthCode(value: string): PaddleAuthCode {
  return value as any;
}

export type MailChimpApiKey = Opaque<string, "MailChimpApiKey">;
export function MailChimpApiKey(value: string): MailChimpApiKey {
  return value as any;
}

export type SubscriptionNonce = Opaque<string, "SubscriptionNonce">;
export function SubscriptionNonce(value: string): SubscriptionNonce {
  return value as any;
}

export type Uuid = Opaque<string, "Uuid">;
export function Uuid(value: string): Uuid {
  return value as any;
}

export type FileListShard = Opaque<number, "FileListShard">;
export function FileListShard(value: number): FileListShard {
  return value as any;
}

export type ProcessId = Opaque<number, "ProcessId">;
export function ProcessId(value: number): ProcessId {
  return value as any;
}

/**
 * SPID and TID (Thread ID) are used interchangeably on Linux, and describe thread IDs. Each process has at least one
 * thread, and that thread has a SPID (thread ID) that matches the process's PID. The process may then have additional SPIDs/threads.
 */
export type SPID = Opaque<number, "SPID">;
export function SPID(value: number): SPID {
  return value as any;
}

export type FileListShardCount = Opaque<number, "FileListShardCount">;
export function FileListShardCount(value: number): FileListShardCount {
  return value as any;
}

export type StorageEventShard = Opaque<number, "StorageEventShard">;
export function StorageEventShard(value: number): StorageEventShard {
  return value as any;
}

export type StorageEventShardCount = Opaque<number, "StorageEventShardCount">;
export function StorageEventShardCount(value: number): StorageEventShardCount {
  return value as any;
}

export type TransformationEventShard = Opaque<number, "TransformationEventShard">;
export function TransformationEventShard(value: number): TransformationEventShard {
  return value as any;
}

export type TransformationEventShardCount = Opaque<number, "TransformationEventShardCount">;
export function TransformationEventShardCount(value: number): TransformationEventShardCount {
  return value as any;
}

export type PostHogApiKey = Opaque<string, "PostHogApiKey">;
export function PostHogApiKey(value: string): PostHogApiKey {
  return value as any;
}

export type DownloadEventShard = Opaque<number, "DownloadEventShard">;
export function DownloadEventShard(value: number): DownloadEventShard {
  return value as any;
}

export type DownloadEventShardCount = Opaque<number, "DownloadEventShardCount">;
export function DownloadEventShardCount(value: number): DownloadEventShardCount {
  return value as any;
}

export type EventType = Opaque<string, "EventType">;
export function EventType(value: string): EventType {
  return value as any;
}

export type UploadPartIndex = Opaque<number, "UploadPartIndex">;
export function UploadPartIndex(value: number): UploadPartIndex {
  return value as any;
}

export type TransformerAccessToken = Opaque<string, "TransformerAccessToken">;
export function TransformerAccessToken(value: string): TransformerAccessToken {
  return value as any;
}

// Contains a '/' prefix.
export type TransformationArtifactPath = Opaque<string, "TransformationArtifactPath">;
export function TransformationArtifactPath(value: string): TransformationArtifactPath {
  return value as any;
}

export type RequestId = Opaque<string, "RequestId">;
export function RequestId(value: string): RequestId {
  return value as any;
}

export type TransformationFunctionSignature = Opaque<string, "TransformationFunctionSignature">;
export function TransformationFunctionSignature(value: string): TransformationFunctionSignature {
  return value as any;
}

export type TrustZoneId = Opaque<string, "TrustZoneId">;
export function TrustZoneId(value: string): TrustZoneId {
  return value as any;
}

export type LambdaFunctionName = Opaque<string, "LambdaFunctionName">;
export function LambdaFunctionName(value: string): LambdaFunctionName {
  return value as any;
}

export type NpmPackageName = Opaque<string, "NpmPackageName">;
export function NpmPackageName(value: string): NpmPackageName {
  return value as any;
}

export type NpmPackageVersion = Opaque<string, "NpmPackageVersion">;
export function NpmPackageVersion(value: string): NpmPackageVersion {
  return value as any;
}

export type NpmUsername = Opaque<string, "NpmUsername">;
export function NpmUsername(value: string): NpmUsername {
  return value as any;
}

export type CacheId = Opaque<string, "CacheId">;
export function CacheId(value: string): CacheId {
  return value as any;
}

export type Stage = Opaque<string, "Stage">;
export function Stage(value: string): Stage {
  return value as any;
}

export type RoleArn = Opaque<string, "RoleArn">;
export function RoleArn(value: string): RoleArn {
  return value as any;
}

export type SesEmailDomainArn = Opaque<string, "SesEmailDomainArn">;
export function SesEmailDomainArn(value: string): SesEmailDomainArn {
  return value as any;
}

export type DynamoDBTableName = Opaque<string, "DynamoDBTableName">;
export function DynamoDBTableName(value: string): DynamoDBTableName {
  return value as any;
}

export type KinesisShardId = Opaque<string, "KinesisShardId">;
export function KinesisShardId(value: string): KinesisShardId {
  return value as any;
}

export type KinesisPartitionKey = Opaque<string, "KinesisPartitionKey">;
export function KinesisPartitionKey(value: string): KinesisPartitionKey {
  return value as any;
}

export type KinesisStreamName = Opaque<string, "KinesisStreamName">;
export function KinesisStreamName(value: string): KinesisStreamName {
  return value as any;
}

export type SQSQueueUrl = Opaque<string, "SQSQueueUrl">;
export function SQSQueueUrl(value: string): SQSQueueUrl {
  return value as any;
}

export type CloudWatchLogEventId = Opaque<string, "CloudWatchLogEventId">;
export function CloudWatchLogEventId(value: string): CloudWatchLogEventId {
  return value as any;
}

export type SentryDsn = Opaque<string, "SentryDsn">;
export function SentryDsn(value: string): SentryDsn {
  return value as any;
}

export type AwsRegion = Opaque<string, "AwsRegion">;
export function AwsRegion(value: string): AwsRegion {
  return value as any;
}

export type AwsAccessKey = Opaque<string, "AwsAccessKey">;
export function AwsAccessKey(value: string): AwsAccessKey {
  return value as any;
}

export type AwsSecretKey = Opaque<string, "AwsSecretKey">;
export function AwsSecretKey(value: string): AwsSecretKey {
  return value as any;
}

export type S3BucketName = Opaque<string, "S3BucketName">;
export function S3BucketName(value: string): S3BucketName {
  return value as any;
}

export type CognitoUserPoolId = Opaque<string, "CognitoUserPoolId">;
export function CognitoUserPoolId(value: string): CognitoUserPoolId {
  return value as any;
}

export type CognitoUserPoolClientId = Opaque<string, "CognitoUserPoolClientId">;
export function CognitoUserPoolClientId(value: string): CognitoUserPoolClientId {
  return value as any;
}

// Event IDs are ULIDs (i.e. time-ordered)
export type EventId = Opaque<string, "EventId">;
export function EventId(value: string): EventId {
  return value as any;
}

/**
 * Reflects the order in which an event was published onto a stream. Any two events can be causally ordered based on
 * this number (unlike the event ID, which is granular only down to millisecond-level, and may drift between servers).
 * We store as 'string' instead of 'BigInt' since the latter causes serialization issues in various libraries (e.g. Pino
 * and possibly DynamoDB)... **but for comparison between serial numbers you should first convert to BigInt**
 */
export type EventSequenceNumber = Opaque<string, "EventSequenceNumber">;
export function EventSequenceNumber(value: string): EventSequenceNumber {
  return value as any;
}

export type OriginalFileName = Opaque<string, "OriginalFileName">;
export function OriginalFileName(value: string): OriginalFileName {
  return value as any;
}

/**
 * Do NOT start with a '/' prefix.
 */
export type S3ObjectKey = Opaque<string, "S3ObjectKey">;
export function S3ObjectKey(value: string): S3ObjectKey {
  return value as any;
}

/**
 * This is NOT a full S3 object key!
 *
 * This is instead something we've invented:
 *
 * - If the object is a file, the value will not contain any "/" characters, e.g. "foo".
 *
 * - If the object is a folder, the value will contain a "/" suffix and no other "/" characters, e.g. "foo/"
 *
 * - If the object is a nameless folder, the value will be "/".
 */
export type S3ObjectName = Opaque<string, "S3ObjectName">;
export function S3ObjectName(value: string): S3ObjectName {
  return value as any;
}

export type S3MultipartUploadId = Opaque<string, "S3MultipartUploadId">;
export function S3MultipartUploadId(value: string): S3MultipartUploadId {
  return value as any;
}

export type UploadId = Opaque<string, "UploadId">;
export function UploadId(value: string): UploadId {
  return value as any;
}

/**
 * 1 = 100%
 */
export type Factor = Opaque<number, "Factor">;
export function Factor(value: number): Factor {
  return value as any;
}

/**
 * ETags are always wrapped in quotes (https://tools.ietf.org/html/rfc2616#section-14.19). However, we don't enforce
 * this and we don't inspect this. In this codebase, ETags are always given to us by S3: we don't try to remove these
 * quotes to save 4 characters of storage per file (since in JSON this serializes to "\"xyz\"") because unwrapping and
 * then re-wrapping again (for the call back to S3) creates a larger surface area for bugs: we simply tunnel what S3
 * provides to the client, to our API, then to S3 again from the API's side, as a blackbox parameter.
 */
export type ETag = Opaque<string, "ETag">;
export function ETag(value: string): ETag {
  return value as any;
}

/**
 * URL for an http(s) resource.
 */
export type HttpUrl = Opaque<string, "HttpUrl">;
export function HttpUrl(value: string): HttpUrl {
  return value as any;
}

export type HttpUrlNoHost = Opaque<string, "HttpUrlNoHost">;
export function HttpUrlNoHost(value: string): HttpUrlNoHost {
  return value as any;
}

/**
 * URL for an http(s) resource WITHOUT the path, e.g. "https://upload.io"
 */
export type HttpOrigin = Opaque<string, "HttpOrigin">;
export function HttpOrigin(value: string): HttpOrigin {
  return value as any;
}

export type EmailAddress = Opaque<string, "EmailAddress">;
export function EmailAddress(value: string): EmailAddress {
  return value as any;
}

export type FirstName = Opaque<string, "FirstName">;
export function FirstName(value: string): FirstName {
  return value as any;
}

export type LastName = Opaque<string, "LastName">;
export function LastName(value: string): LastName {
  return value as any;
}

export type FSFilePath = Opaque<string, "FSFilePath">;
export function FSFilePath(value: string): FSFilePath {
  return value as any;
}

export type TcpPort = Opaque<number, "TcpPort">;
export function TcpPort(value: number): TcpPort {
  return value as any;
}

export type DistributedLockId = Opaque<string, "DistributedLockId">;
export function DistributedLockId(value: string): DistributedLockId {
  return value as any;
}

export type DistributedLockOwner = Opaque<string, "DistributedLockOwner">;
export function DistributedLockOwner(value: string): DistributedLockOwner {
  return value as any;
}

/**
 * Unique within the scope of a single account only. Not guaranteed to be unique globally. Does not contain the
 * account ID as a prefix, so must accompany an account ID for the purpose of identifying a public key.
 */
export type PublicKeyId = Opaque<string, "PublicKeyId">;
export function PublicKeyId(value: string): PublicKeyId {
  return value as any;
}

export type PublicKeyName = Opaque<string, "PublicKeyName">;
export function PublicKeyName(value: string): PublicKeyName {
  return value as any;
}

/**
 * Public key in PEM format: "-----BEGIN PUBLIC KEY-----...-----END PUBLIC KEY-----"
 */
export type PublicKeyPEM = Opaque<string, "PublicKeyPEM">;
export function PublicKeyPEM(value: string): PublicKeyPEM {
  return value as any;
}

/**
 * Public key in PEM format: "-----BEGIN RSA PRIVATE KEY-----...-----END RSA PRIVATE KEY-----"
 */
export type PrivateKeyPEM = Opaque<string, "PrivateKeyPEM">;
export function PrivateKeyPEM(value: string): PrivateKeyPEM {
  return value as any;
}

export type Base64 = Opaque<string, "Base64">;
export function Base64(value: string): Base64 {
  return value as any;
}

/**
 * Unique within the scope of a single account only. Not guaranteed to be unique globally. Does not contain the
 * account ID as a prefix, so must accompany an account ID for the purpose of identifying a transformation.
 */
export type TopLevelFolderId = Opaque<number, "TopLevelFolderId">;
export function TopLevelFolderId(value: number): TopLevelFolderId {
  return value as any;
}

/**
 * Unique within the scope of a single account only. Not guaranteed to be unique globally. Does not contain the
 * account ID as a prefix, so must accompany an account ID for the purpose of identifying a transformation.
 */
export type TopLevelFolderUrlSlug = Opaque<string, "TopLevelFolderUrlSlug">;
export function TopLevelFolderUrlSlug(value: string): TopLevelFolderUrlSlug {
  return value as any;
}

export type TopLevelFolderName = Opaque<string, "TopLevelFolderName">;
export function TopLevelFolderName(value: string): TopLevelFolderName {
  return value as any;
}
export type TopLevelFolderDescription = Opaque<string, "TopLevelFolderDescription">;
export function TopLevelFolderDescription(value: string): TopLevelFolderDescription {
  return value as any;
}

export type ListFolderChildrenCursor = Opaque<string, "ListFolderChildrenCursor">;
export function ListFolderChildrenCursor(value: string): ListFolderChildrenCursor {
  return value as any;
}

export type ListFolderChildrenLimit = Opaque<number, "ListFolderChildrenLimit">;
export function ListFolderChildrenLimit(value: number): ListFolderChildrenLimit {
  return value as any;
}

export type FolderDescription = Opaque<string, "FolderDescription">;
export function FolderDescription(value: string): FolderDescription {
  return value as any;
}

/**
 * The full path, that appears after target ID, for a target request. It includes the file path (to request from the
 * target -- called the 'UpstreamOriginalFilePath' below), the transformation slug, and optionally the path artifact.
 */
export type FolderRequestPath = Opaque<string, "FolderRequestPath">;
export function FolderRequestPath(value: string): FolderRequestPath {
  return value as any;
}

/**
 * May be empty string `""`.
 * Cannot contain `/`.
 */
export type FolderName = Opaque<string, "FolderName">;
export function FolderName(value: string): FolderName {
  return value as any;
}

/**
 * May be empty string "".
 * Cannot contain "/".
 */
export type FileName = Opaque<string, "FileName">;
export function FileName(value: string): FileName {
  return value as any;
}

/**
 * If starts with '/' has the same semantics and treatment as a 'FilePath'.
 *
 * Else must be combined with a base folder through 'FolderPathUtils.combine'.
 *
 * CAVEAT: this will cause problems as it will prevent you referencing a file from a sibling unnamed directory (as the
 * relative path will begin with a '/' and will look like an absolute path...). This will just have to be a limitation,
 * in that certain parts of the Upload.io system will mistreat these edge-case relative paths as absolute paths (e.g.
 * image paths for watermarks). In practice, this should be limited to a few places, and we can always warn against
 * unnamed folders, saying they have limited support.
 *
 * May contain '..' although these don't traverse and are just part of the file path (like how S3 keys can contain '..').
 */
export type RelativeOrAbsoluteFilePath = Opaque<string, "RelativeOrAbsoluteFilePath">;
export function RelativeOrAbsoluteFilePath(value: string): RelativeOrAbsoluteFilePath {
  return value as any;
}

export type RelativeOrAbsoluteFolderPath = Opaque<string, "RelativeOrAbsoluteFolderPath">;
export function RelativeOrAbsoluteFolderPath(value: string): RelativeOrAbsoluteFolderPath {
  return value as any;
}

/**
 * Excludes leading "/".
 *
 * May contain other "/" including a trailing "/" if this refers to a nameless file.
 *
 * See 'RelativeOrAbsoluteFilePath' for caveats and other semantics.
 */
export type RelativeFilePath = Opaque<string, "RelativeFilePath">;
export function RelativeFilePath(value: string): RelativeFilePath {
  return value as any;
}

/**
 * Path to a folder under management by Upload (either stored on Upload, or externally via another storage layer).
 *
 * Excludes trailing "/" (unless the leaf folder name is empty string, or this is the root path "/").
 * Includes leading "/"
 *
 * See 'FileOrFolderPath' below for examples.
 */
export type FolderPath = Opaque<string, "FolderPath">;
export function FolderPath(value: string): FolderPath {
  return value as any;
}

export type ApiKeyId = Opaque<string, "ApiKeyId">;
export function ApiKeyId(value: string): ApiKeyId {
  return value as any;
}

/**
 * Path to a file under management by Upload (either stored on Upload, or externally via another storage layer).
 *
 * See 'FileOrFolderPath' below for examples.
 */
export type FilePath = Opaque<string, "FilePath">;
export function FilePath(value: string): FilePath {
  return value as any;
}

/**
 * Path to a file or folder under management by Upload (either stored on Upload, or externally via another storage layer).
 *
 * Excludes:
 * - Account ID
 * - Transformation slug.
 *
 * Includes:
 * - Top-level folder (e.g. "/uploads").
 * - The "/" prefix.
 *
 * Examples:
 * - "/uploads/abc1234abc1234fG"                                -- File uploaded to default location
 * - "/example/folder/"                                         -- Apex (i.e. empty string) file name
 * - "/example/path//with/../some?/special:characters"          -- File name with special characters
 * - "/web/https://images.unsplash.com/image.jpg?w=800&h=600"   -- Web folder file name
 */
export type FileOrFolderPath = Opaque<string, "FileOrFolderPath">;
export function FileOrFolderPath(value: string): FileOrFolderPath {
  return value as any;
}

/**
 * Does NOT contain a "/" prefix.
 * Does NOT include the top-level folder.
 * MAY contain '?' and query params (e.g. for upstream URLs).
 * MAY contain ':' (and in fact may represent an entire URL)
 */
export type FolderApiFileUrl = Opaque<string, "FolderApiFileUrl">;
export function FolderApiFileUrl(value: string): FolderApiFileUrl {
  return value as any;
}

/**
 * Unique within the scope of a single account only. Not guaranteed to be unique globally. Does not contain the
 * account ID as a prefix, so must accompany an account ID for the purpose of identifying a transformation.
 */
export type TransformationDefinitionId = Opaque<string, "TransformationDefinitionId">;
export function TransformationDefinitionId(value: string): TransformationDefinitionId {
  return value as any;
}

export type TransformationDescription = Opaque<string, "TransformationDescription">;
export function TransformationDescription(value: string): TransformationDescription {
  return value as any;
}

// Can be 'raw' to indicate no transformation.
export type TransformationUrlSlug = Opaque<string, "TransformationUrlSlug">;
export function TransformationUrlSlug(value: string): TransformationUrlSlug {
  return value as any;
}

// If ends with a '*', represents a prefix.
export type TransformationUrlSlugPattern = Opaque<string, "TransformationUrlSlugPattern">;
export function TransformationUrlSlugPattern(value: string): TransformationUrlSlugPattern {
  return value as any;
}

export type RuleDescription = Opaque<string, "RuleDescription">;
export function RuleDescription(value: string): RuleDescription {
  return value as any;
}

export type RuleName = Opaque<string, "RuleName">;
export function RuleName(value: string): RuleName {
  return value as any;
}

export type RuleErrorMessage = Opaque<string, "RuleErrorMessage">;
export function RuleErrorMessage(value: string): RuleErrorMessage {
  return value as any;
}

export type FileExpiredMessage = Opaque<string, "FileExpiredMessage">;
export function FileExpiredMessage(value: string): FileExpiredMessage {
  return value as any;
}

export type MimeScope = Opaque<string, "MimeScope">;
export function MimeScope(value: string): MimeScope {
  return value as any;
}

export type TagScope = Opaque<string, "TagScope">;
export function TagScope(value: string): TagScope {
  return value as any;
}

export type MimeType = Opaque<string, "MimeType">;
export function MimeType(value: string): MimeType {
  return value as any;
}

export type FileTagName = Opaque<string, "FileTagName">;
export function FileTagName(value: string): FileTagName {
  return value as any;
}

export type WarmTransformationFunctionLimit = Opaque<number, "WarmTransformationFunctionLimit">;
export function WarmTransformationFunctionLimit(value: number): WarmTransformationFunctionLimit {
  return value as any;
}

export type HotTransformationFunctionLimit = Opaque<number, "HotTransformationFunctionLimit">;
export function HotTransformationFunctionLimit(value: number): HotTransformationFunctionLimit {
  return value as any;
}

// Why not use 'bigint'? Because DynamoDB stores all numbers as the 'N' type and will deserialize to either 'number' or
// 'bigint' depending on the size of the integer. This means on deserializing values back out of DynamoDB, some
// instances of 'BigSizeInBytes' will be 'number' and others will be 'bigint', depending on their value. This is
// problematic, because you cannot perform 'number + bigint' (you get the error "Cannot mix BigInt and other types, use
// explicit conversions"). Therefore, we pass around as 'string' to ensure DynamoDB doesn't implicitly change its type
// between two different types at runtime.
export type BigSizeInBytes = Opaque<string, "BigSizeInBytes">;
export function BigSizeInBytes(value: string): BigSizeInBytes {
  return value as any;
}

/**
 * An 'object author' is the name of the application that created/updated the object.
 */
export type ApplicationName = Opaque<string, "ApplicationName">;
export function ApplicationName(value: string): ApplicationName {
  return value as any;
}

/**
 * An 'object author version' is the version the author application was at (see above) when it performed the create/update.
 */
export type ApplicationVersion = SemanticVersion;

export type SemanticVersion = Opaque<string, "SemanticVersion">;
export function SemanticVersion(value: string): SemanticVersion {
  return value as any;
}

export type ObjectVersion = Opaque<number, "ObjectVersion">;
export function ObjectVersion(value: number): ObjectVersion {
  return value as any;
}
export function ObjectVersionInc(value: ObjectVersion): ObjectVersion {
  return ObjectVersion(value.valueOf() + 1);
}

export type Jwt = Opaque<string, "Jwt">;
export function Jwt(value: string): Jwt {
  return value as any;
}

export type EpochMillis = Opaque<number, "EpochMillis">;
export function EpochMillis(value: number): EpochMillis {
  return value as any;
}

export type EpochSeconds = Opaque<number, "EpochSeconds">;
export function EpochSeconds(value: number): EpochSeconds {
  return value as any;
}

export type EpochDays = Opaque<number, "EpochDays">;
export function EpochDays(value: number): EpochDays {
  return value as any;
}

export type EpochMonths = Opaque<number, "EpochMonths">;
export function EpochMonths(value: number): EpochMonths {
  return value as any;
}

export type PageSize = Opaque<number, "PageSize">;
export function PageSize(value: number): PageSize {
  return value as any;
}

export type ContinuationToken = Opaque<string, "ContinuationToken">;
export function ContinuationToken(value: string): ContinuationToken {
  return value as any;
}

export type ApiKeyDescription = Opaque<string, "ApiKeyDescription">;
export function ApiKeyDescription(value: string): ApiKeyDescription {
  return value as any;
}

export type PublicApiKey = Opaque<string, "PublicApiKey">;
export function PublicApiKey(value: string): PublicApiKey {
  return value as any;
}

export type SecretApiKey = Opaque<string, "SecretApiKey">;
export function SecretApiKey(value: string): SecretApiKey {
  return value as any;
}

// Can be "localhost" too.
export type DomainName = Opaque<string, "DomainName">;
export function DomainName(value: string): DomainName {
  return value as any;
}

// Only registerable domains, i.e. cannot be "localhost".
export type CustomDomainName = Opaque<string, "CustomDomainName">;
export function CustomDomainName(value: string): CustomDomainName {
  return value as any;
}

export type CustomDomainNameTargetDomain = Opaque<string, "CustomDomainNameTargetDomain">;
export function CustomDomainNameTargetDomain(value: string): CustomDomainNameTargetDomain {
  return value as any;
}

export type PluginDescription = Opaque<string, "PluginDescription">;
export function PluginDescription(value: string): PluginDescription {
  return value as any;
}

export type TemplateCategory = Opaque<string, "TemplateCategory">;
export function TemplateCategory(value: string): TemplateCategory {
  return value as any;
}

export type TemplateKey = Opaque<string, "TemplateKey">;
export function TemplateKey(value: string): TemplateKey {
  return value as any;
}

export type TemplateName = Opaque<string, "TemplateName">;
export function TemplateName(value: string): TemplateName {
  return value as any;
}

export type TemplateDescription = Opaque<string, "TemplateDescription">;
export function TemplateDescription(value: string): TemplateDescription {
  return value as any;
}

export type TemplateParamDefault = Opaque<string, "TemplateParamDefault">;
export function TemplateParamDefault(value: string): TemplateParamDefault {
  return value as any;
}

export type TemplateParamDescription = Opaque<string, "TemplateParamDescription">;
export function TemplateParamDescription(value: string): TemplateParamDescription {
  return value as any;
}

export type TemplateParamLabel = Opaque<string, "TemplateParamLabel">;
export function TemplateParamLabel(value: string): TemplateParamLabel {
  return value as any;
}

export type TemplateParamPlaceholder = Opaque<string, "TemplateParamPlaceholder">;
export function TemplateParamPlaceholder(value: string): TemplateParamPlaceholder {
  return value as any;
}

export type IpAddressV4 = Opaque<string, "IpAddressV4">;
export function IpAddressV4(value: string): IpAddressV4 {
  return value as any;
}
