import { JsonValue } from "@upload-io/upload-shared-iso";
import { JSONSchema7 } from "json-schema";
import { OrderSensitiveQueryParams } from "@upload-io/upload-shared-iso/common/model/OrderSensitiveQueryParams";
import { QueryStringToJsonFunction } from "@upload-io/upload-shared-iso/plugins/qsToJsonMapper/utils/QueryStringToJsonFunction";
import { JsonToQueryStringFunction } from "@upload-io/upload-shared-iso/plugins/qsToJsonMapper/utils/JsonToQueryStringFunction";
import { QueryParamDefinitionMap } from "@upload-io/upload-shared-iso/plugins/qsToJsonMapper/model/QueryParamDefinitionMap";
import { QueryParamDefinitionFactory } from "@upload-io/upload-shared-iso/plugins/qsToJsonMapper/utils/QueryParamDefinitionFactory";
import $RefParser from "@apidevtools/json-schema-ref-parser";
import { QueryStringToJsonResult } from "@upload-io/upload-shared-iso/plugins/qsToJsonMapper/model/QueryStringToJsonResult";

/**
 * Creates a query-string oriented API for any JSON schema.
 *
 * The API is two-way:
 *
 * -   A query-string can be converted to a JSON object (that is valid per the given schema)
 *
 * -   A JSON object can be converted into a query string.
 */
export class QueryStringToJsonMapper {
  private readonly toJson: QueryStringToJsonFunction;
  private readonly toQueryString: JsonToQueryStringFunction;

  constructor(schema: JSONSchema7, paramDefinitions: QueryParamDefinitionMap) {
    this.toJson = new QueryStringToJsonFunction(schema, paramDefinitions);
    this.toQueryString = new JsonToQueryStringFunction(schema);
  }

  static async fromJsonSchema(rawSchema: JSONSchema7): Promise<QueryStringToJsonMapper> {
    const schema = (await $RefParser.dereference(rawSchema)) as JSONSchema7;
    const paramDefinitions = QueryParamDefinitionFactory.createDefinitions(schema);
    return new QueryStringToJsonMapper(schema, paramDefinitions);
  }

  fromQuerystring(queryString: OrderSensitiveQueryParams): QueryStringToJsonResult {
    return this.toJson.toJson(queryString);
  }

  toQuerystring(params: JsonValue): OrderSensitiveQueryParams {
    return this.toQueryString.toQueryString(params);
  }
}
