import { FieldWithMetadata } from "@upload-io/upload-shared-iso/plugins/qsToJsonMapper/model/ObjectWithMetadata";
import { assertUnreachable, JsonValue } from "@upload-io/upload-shared-iso";
import { sortBy } from "ramda";
import { QueryParamEncoder } from "@upload-io/upload-shared-iso/plugins/qsToJsonMapper/utils/QueryParamEncoder";

export class ObjectRasterizer {
  /**
   * Converts an 'ObjectWithMetadata' to a regular JsonValid (essentially just strips the metadata).
   */
  rasterize({ fieldValue, possibleTypes }: FieldWithMetadata<unknown>): JsonValue {
    switch (fieldValue.type) {
      case "scalar":
        return fieldValue.source === "autofill" || typeof fieldValue.value !== "string"
          ? fieldValue.value
          : QueryParamEncoder.decodeFromQueryString(fieldValue.value, possibleTypes[0].schema);
      case "object":
        return Object.fromEntries(
          possibleTypes[0].children.map((property): [string, JsonValue] => [
            property.fieldName,
            this.rasterize(property)
          ])
        );
      case "array":
        return sortBy(
          x => x[0],
          possibleTypes[0].children.map((property): [number, JsonValue] => [
            parseInt(property.fieldName),
            this.rasterize(property)
          ])
        ).map(x => x[1]);

      default:
        assertUnreachable(fieldValue);
    }
  }
}
