/**
 * IMPORTANT:
 * - This is a special-case encoder, specifically for use with the JSON<->QueryString framework we've built.
 * - It does not actually URL encode.
 * - Instead, it prepares/unpacks data for/from the querystring, based on the JSON Schema of the object it's going to/coming from.
 */
import { JSONSchema7 } from "json-schema";

export class QueryParamEncoder {
  static encodeForQueryString(value: string, schema: JSONSchema7): string {
    if (QueryParamEncoder.fieldStartsWithHash(schema) && value.startsWith("#")) {
      // Trim the preceding "#" if the schema says it always starts with a hash: we re-add it on decode.
      // This is because hashes obviously can't be used in query strings without URL encoding, which is ugly, especially
      // if we're frequently demonstrating using RGB/hex-codes in URLs.
      return value.substring(1);
    }

    return value;
  }

  static decodeFromQueryString(value: string, schema: JSONSchema7): string {
    if (QueryParamEncoder.fieldStartsWithHash(schema) && !value.startsWith("#")) {
      return `#${value}`;
    }

    return value;
  }

  private static fieldStartsWithHash(schema: JSONSchema7): boolean {
    return schema.pattern?.startsWith("^#") === true;
  }
}
