import { EpochMillis, maybe } from "@upload-io/upload-shared-iso/common/model/Primitives";
import { EpochMillisUnboxed } from "@upload-io/upload-shared-iso/common/model/UnboxedPrimitives";
import { SizeInBytesUnboxed, SizeInBytes } from "@upload-io/upload-shared-iso-lite";

export interface StorageStatisticsSegmentDto {
  firstFileCreated: EpochMillis | undefined;
  lastFileCreated: EpochMillis | undefined;
  totalFileCount: number;
  totalFileCountMonotonic: number | undefined;
  totalFileSize: SizeInBytes;
  totalFileSizeMonotonic: SizeInBytes | undefined;
}

export interface StorageStatisticsSegmentUnboxedDto {
  firstFileCreated: EpochMillisUnboxed | null;
  lastFileCreated: EpochMillisUnboxed | null;
  totalFileCount: number;
  totalFileCountMonotonic: number | null;
  totalFileSize: SizeInBytesUnboxed;
  totalFileSizeMonotonic: SizeInBytesUnboxed | null;
}

function reduceOpt(
  left: number | undefined,
  right: number | undefined,
  reduce: (l: number, r: number) => number
): number | undefined {
  if (left === undefined) {
    return right;
  }
  if (right === undefined) {
    return left;
  }
  return reduce(left, right);
}

export namespace StorageStatisticsSegmentDto {
  export const empty: StorageStatisticsSegmentDto = {
    firstFileCreated: undefined,
    lastFileCreated: undefined,
    totalFileCount: 0,
    totalFileCountMonotonic: 0,
    totalFileSize: SizeInBytes(0),
    totalFileSizeMonotonic: SizeInBytes(0)
  };

  export function combine(
    left: StorageStatisticsSegmentDto,
    right: StorageStatisticsSegmentDto
  ): StorageStatisticsSegmentDto {
    return {
      firstFileCreated: maybe(EpochMillis, reduceOpt(left.firstFileCreated, right.firstFileCreated, Math.min)),
      lastFileCreated: maybe(EpochMillis, reduceOpt(left.lastFileCreated, right.lastFileCreated, Math.max)),
      totalFileCount: left.totalFileCount + right.totalFileCount,
      totalFileCountMonotonic: (left.totalFileCountMonotonic ?? 0) + (right.totalFileCountMonotonic ?? 0),
      totalFileSize: SizeInBytes(left.totalFileSize.valueOf() + right.totalFileSize.valueOf()),
      totalFileSizeMonotonic: SizeInBytes(
        (left.totalFileSizeMonotonic?.valueOf() ?? 0) + (right.totalFileSizeMonotonic?.valueOf() ?? 0)
      )
    };
  }
}
